<?php
namespace Drupal\data_api;

/**
 * Class Data
 *
 * Provides a Drupal implementation, which handles language translation on
 * entities and catches errors throw in entity classes.
 *
 * Global Drupal functions have been wrapped in class methods for encapsulation
 * reasons.  Take a look at the class DataMock for how you would decouple this
 * class from Drupal for unit testing, by extending those methods.
 *
 * @package Drupal\data_api
 */
class Data extends \AKlump\Data\Data
{
    protected $entityType = null;

    /**
     * @inheritdoc
     */
    public function get($subject, $path, $defaultValue = null, $valueCallback = null)
    {
        $this->cacheSet(__FUNCTION__, $subject, $path, $defaultValue, $valueCallback);
        if (empty($subject)) {
            return $this->postGet($defaultValue, $defaultValue, $valueCallback, false);
        }

        // This will make sure $path is an array.
        $this->validate($subject, $path);

        // When we know the entity type, we can use Drupal's field api functions.
        // This will take care of the translation.
        if (empty($this->cache['get']['level'])
            && ($field_name = reset($path))
            && ($bundle = $this->getBundleType($subject))
            && $this->isField($bundle, $field_name)
        ) {
            array_shift($path);
            if (!($subject = $this->field_get_items($this->getEntityType(), $subject, $field_name))) {
                return $this->postGet($defaultValue, $defaultValue, $valueCallback, false);
            }
            elseif (count($path) === 0) {
                return $this->postGet($subject, $defaultValue, $valueCallback, true);
            }
        }

        try {
            $return = parent::get($subject, $path, $defaultValue, $valueCallback);
        } catch (\Exception $exception) {
            watchdog_exception('data_api', $exception);
            $return = $defaultValue;
        }

        return $return;
    }

    public function set(&$subject, $path = null, $value = null, $childTemplate = null)
    {
        if ($this->cache['carry']['abort']) {
            return $this;
        }

        $this->writeArgHandler(func_num_args());
        $this->useCarry($path, $value);
        $this->cacheSet(__FUNCTION__, $subject, $path, $value, $childTemplate);
        $this->validate($subject, $path);

        // Determine if we need to add the language component to the path.
        if ($value && empty($this->cache['set']['level']) && is_object($subject) && ($field_name = reset($path))) {
            $temp = clone $subject;;
            $temp->{$field_name} = isset($temp->{$field_name}) ? $temp->{$field_name} : $childTemplate;
            if (($bundle = $this->getBundleType($temp))
                && $this->isField($bundle, $field_name)
            ) {
                $langcode = $this->field_language($this->getEntityType(), $subject, $field_name);
                if (empty($path[1]) || $path[1] !== $langcode) {
                    array_splice($path, 1, 0, array($langcode));
                    $childTemplate = array();
                }
            }
        };

        try {
            $return = parent::set($subject, $path, $value, $childTemplate);
        } catch (\Exception $exception) {
            watchdog_exception('data_api', $exception);
            $return = $this;
        }

        return $return;
    }

    /**
     * @return string|null
     */
    public function getEntityType()
    {
        return $this->entityType;
    }

    /**
     * @param string $entity_type
     *
     * @return $this
     */
    public function setEntityType($entity_type)
    {
        $this->entityType = $entity_type;

        return $this;
    }

    /**
     * Return the bundle type of the entity.
     *
     * @param string $subject
     *
     * @return null|bool
     */
    protected function getBundleType($subject)
    {
        $entity_type = $this->getEntityType();
        if (empty($entity_type)) {
            return null;
        }

        // We do not want to throw an exception here.
        try {
            list(, , $bundle) = $this->entity_extract_ids($entity_type, $subject);
        } catch (\Exception $e) {
            $bundle = null;
        }

        return $bundle;
    }

    /**
     * @codeCoverageIgnore
     */
    protected function entity_extract_ids($entity_type, $entity)
    {
        return entity_extract_ids($entity_type, $entity);
    }

    /**
     * Tests if $field is a field instance on $bundle.
     *
     * @param string $bundle
     * @param string $field
     *
     * @return bool
     */
    protected function isField($bundle, $field)
    {
        return array_key_exists($field, $this->field_info_instances($this->getEntityType(), $bundle));
    }

    /**
     * @codeCoverageIgnore
     */
    protected function field_info_instances($entity_type = null, $bundle_name = null)
    {
        return field_info_instances($entity_type, $bundle_name);
    }

    /**
     * @codeCoverageIgnore
     */
    protected function field_get_items($entity_type, $entity, $field_name, $langcode = null)
    {
        return field_get_items($entity_type, $entity, $field_name, $langcode);
    }

    /**
     * @codeCoverageIgnore
     */
    protected function field_language($entity_type, $entity, $field_name = null, $langcode = null)
    {
        return field_language($entity_type, $entity, $field_name, $langcode);
    }
}
