<?php
namespace Drupal\data_api;

/**
 * Class DataApiTrait
 *
 * Add this trait to classes that need to use Drupal\data_api\Data internally.
 * The using class SHOULD inject a Data object in the constructor as follows:
 * @code
 *   public function __construct(Data $dataApiData)
 *   {
 *     $this->setDataApiData($dataApiData);
 *   }
 * @endcode
 *
 * Notice the reserved properties: g, e, n, u, when you need to use the object
 * in a method, choose the appropriate property by node type and do...
 * @code
 *   ...
 *   $body = $this->n->get($node, 'field_body.0.value');
 *   ...
 * @endcode
 *
 * For static methods you MUST use the data_api() function instead.
 * @code
 *   $body = data_api('node')->get($node, 'field_body.0.value');
 * @endcode
 *
 */
trait DataTrait
{
    protected $dataApiData;

    /**
     * @var Holds a single instance of a global (no entity type) Data object.
     */
    protected $g;

    /**
     * @var Reserved for an entity Data object.  Caller must instantiate with
     *      the correct entity type per situation.
     *
     * @code
     *      $this->e = $this->getDataApiData('commerce_product');
     * @endcode
     */
    protected $e;

    /**
     * @var A node Data object.
     */
    protected $n;

    /**
     * @var A user Data object.
     */
    protected $u;

    /**
     * Return a new instance of a Data object.
     *
     * @param $entity_type Optional.  The entity type if applicable.
     *
     * @return Data
     */
    protected function getDataApiData($entity_type = null)
    {
        if (is_null($entity_type)) {
            return $this->g;
        }

        static $objects = array();
        if (!array_key_exists($entity_type, $objects)) {
            $data = clone $this->g;
            $objects[$entity_type] = $data->setEntityType($entity_type);
        }

        return $objects[$entity_type];
    }

    /**
     * Set the DataApi objects
     *
     * @param mixed $dataApiData
     *
     * @return $this
     */
    public function setDataApiData(Data $dataApiData)
    {
        $this->dataApiData = $dataApiData;
        $this->g = clone $this->dataApiData;
        $this->g->setEntityType(null);
        $this->n = $this->getDataApiData('node');
        $this->u = $this->getDataApiData('user');

        return $this;
    }
}
