<?php
namespace Drupal\data_api;

class DataTest extends \PHPUnit_Framework_TestCase
{


    public function testSetChained()
    {
        $vars = array('do' => 're');
        $this->data->onlyIf($vars, 'do')
                   ->call(function ($value) {
                       return strtoupper($value);
                   })
                   ->set($vars);
        $this->assertSame('RE', $vars['do']);
    }

    public function testSetWhenEntityTypeIsKnownFieldLevelNull()
    {
        global $entity;
        $node = new \stdClass;
        $node->type = 'person';
        $control = clone $node;
        $entity = $control;
        $control->field_body = null;
        $this->data->setEntityType('node');
        $this->data->set($node, 'field_body', null, array());
        $this->assertEquals($control, $node);
    }

    public function testSetEntireArrayWhenFieldIsEmpty()
    {
        global $entity;
        $entity = new \stdClass;
        $entity->type = 'person';
        $control = clone $entity;
        $entity->field_body = array();

        $items = array(
            array('do' => 're'),
            array('mi' => 'fa'),
        );
        $control->field_body['und'] = $items;

        $this->data->setEntityType('node');
        $this->data->set($entity, 'field_body', $items);
        $this->assertEquals($control, $entity);
    }

    public function testSetNoBundleActsLikeRegular()
    {
        global $entity;
        $entity = new \stdClass;
        $control = clone $entity;
        $entity->field_body = array();

        // Notice since we don't have a bundle type, this falls back to normal array.
        $control->field_body[0]['value'] = 'lorem';
        $this->data->setEntityType('node')
                   ->set($entity, 'field_body.0.value', 'lorem', array());
        $this->assertEquals($control, $entity);
    }

    public function testSetWhenEntityTypeIsKnownFieldLevelArray()
    {
        global $entity;
        $node = new \stdClass;
        $node->type = 'person';
        $control = clone $node;
        $entity = $control;
        $control->field_body = array();
        $this->data->setEntityType('node')
                   ->set($node, 'field_body', array(), array());
        $this->assertEquals($control, $node);
    }

    public function testSetWhenEntityTypeIsKnownDeltaLevel()
    {
        global $entity;
        $node = new \stdClass;
        $node->type = 'person';
        $control = clone $node;
        $entity = $control;
        $value = array('value' => 'lorem');
        $control->field_body['und'][0] = $value;
        $this->data->setEntityType('node')
                   ->set($node, 'field_body.0', $value, array());
        $this->assertEquals($control, $node);
    }

    public function testSetWhenEntityTypeIsKnown()
    {
        global $entity;
        $node = new \stdClass;
        $node->type = 'person';
        $control = clone $node;
        $entity = $control;
        $control->field_body['und'][0]['value'] = 'lorem';
        $this->data->setEntityType('node')
                   ->set($node, 'field_body.0.value', 'lorem', array());
        $this->assertEquals($control, $node);
    }

    public function testSetWhenNoEntityType()
    {
        $node = new \stdClass;
        $control = clone $node;
        $control->field_body['und'][0]['value'] = 'lorem';
        $this->data->set($node, 'field_body.und.0.value', 'lorem', array());
        $this->assertEquals($control, $node);
    }

    public function testCallbackFiredWhenSubjectIsEmpty()
    {
        $called = false;
        $this->data->get(null, 'do.re.mi', 'default', function ($value) use (&$called) {
            $called = true;
        });
        $this->assertTrue($called);
    }

    public function testWhenTheFieldHasNoItems()
    {
        global $entity;
        $entity = (object) array(
            "field_name" => null,
            'type'       => 'person',
        );
        $this->data->setEntityType('node')
                   ->get($entity, 'field_name.0.value', 'apple');
        $this->assertSame('apple', $this->data->get($entity, 'field_name.0.value', 'apple'));
    }

    public function testItemLevel()
    {
        global $entity;
        $subject = array('und' => array(array('value' => 'Scott Johnson')));
        $node = (object) array(
            "field_name" => $subject,
            'type'       => 'person',
        );
        $entity = $node;
        $control = $subject['und'][0]['value'];
        $this->data->setEntityType('node');
        $this->assertSame($control, $this->data->get($node, 'field_name.0.value'));

        // This will test the callback and the default.
        $doesExist = null;
        $this->assertSame('JACK', $this->data->get($node, 'field_name.0.value', 'Jack', function ($value, $default, $exists) use (&$doesExist) {
            $doesExist = $exists;

            return strtoupper($default);
        }));
        $this->assertTrue($doesExist);

        // This will test the callback and the default.
        $doesExist = null;
        $this->assertSame('JACK', $this->data->get($node, 'field_name.0.indicator', 'Jack', function ($value, $default, $exists) use (&$doesExist) {
            $doesExist = $exists;

            return strtoupper($default);
        }));
        $this->assertFalse($doesExist);
    }

    public function testItemLevelWithFourPathComponents()
    {
        global $entity;
        $node = (object) array(
            'type' => 'person',
        );

        // This is not feasible in drupal 7, but let's test it anyway.
        $node->field_name['und'][0]['value']['first'] = 'Aaron';

        $entity = $node;
        $this->data->setEntityType('node');
        $this->assertSame('Aaron', $this->data->get($node, 'field_name.0.value.first'));
    }

    public function testDeltaLevel()
    {
        global $entity;
        $subject = array('und' => array(array('value' => 'Scott Johnson')));
        $node = (object) array(
            "field_name" => $subject,
            'type'       => 'person',
        );
        $entity = $node;
        $control = $subject['und'][0];
        $this->data->setEntityType('node');
        $this->assertSame($control, $this->data->get($node, 'field_name.0'));
    }

    public function testDeltaLevelCallback()
    {
        global $entity;
        $subject = array('und' => array(array('value' => 'Scott Johnson')));
        $node = (object) array(
            "field_name" => $subject,
            'type'       => 'person',
        );
        $entity = $node;
        $control = $subject['und'][0];
        $this->data->setEntityType('node');

        // Test the callback
        $this->assertSame(json_encode($control), $this->data->get($node, 'field_name.0', null, function ($value) {
            return json_encode($value);
        }));
    }

    public function testFieldLevel()
    {
        global $entity;
        $node = (object) array(
            'type' => 'person',
        );
        $node->field_name['und'][0]['value'] = 'Scott Johnson';
        $entity = $node;
        $control = $node->field_name['und'];
        $this->data->setEntityType('node');
        $this->assertSame($control, $this->data->get($node, 'field_name'));

        // Test the callback
        $this->assertSame(json_encode($control), $this->data->get($node, 'field_name', null, function ($value) {
            return json_encode($value);
        }));
    }

    public function testEmptySubject()
    {
        $this->assertSame('froggy', $this->data->get(array(), 'field', 'froggy'));
    }

    public function testNid()
    {
        global $entity;
        $subject = (object) array('nid' => 27);
        $entity = $subject;
        $this->assertSame(27, $this->data->setEntityType('node')
                                         ->get($subject, 'nid'));
    }

    public function testNidNoEntityType()
    {
        global $entity;
        $subject = (object) array('nid' => 27);
        $entity = $subject;
        $this->assertSame(27, $this->data->get($subject, 'nid'));
    }

    public function setUp()
    {
        $this->data = new DataMock;
    }
}
